#!/usr/bin/env python
# -*- coding: iso-8859-15 -*-
#-------------------------------python-----------------------colors.py--#
#                                                                       #
#                        Colors for pygraph                             #
#                                                                       #
# Copyright (c) 2011 Daniele Zambelli
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Function forn colors managing
"""

#import sys, os
import pygrapherror as pyerr
import re

_colbyname = {}
_colbycode = {}

_RERGB = re.compile('\#((\d|[a-f]){2}){3}$')

if __name__ == '__main__':
  colorfile = 'rgb.txt'
else:
  colorfile = __file__[:__file__.index(__name__)]+'rgb.txt'

for line in open(colorfile): #file('rgb.txt').readlines():
  line = line.strip()
  if line and line[0] != '#':
    rgb, name = line.split('\t\t')
    rgb = rgb.strip()
    r_c, g_c, b_c = [int(e) for e in rgb.split(' ') if e]
    code = "#{0:02x}{1:02x}{2:02x}".format(r_c, g_c, b_c)
    _colbyname[name] = code
    if not code in _colbycode:
      _colbycode[code] = name
##print(len(_colbyname))
##print()
##print(len(_colbycode))

def codecolor(arg):
  """Return color code.
  
  Examples:
  >>> print(codecolor(''))
  <BLANKLINE>
  >>> print(codecolor('orange'))
  #ffa500
  >>> print(codecolor('#c0c0c0'))
  #c0c0c0
  >>> print(codecolor((0.2, 0.8, 0.55)))
  #33cc8c
  >>> print(codecolor((200, 100, 55)))
  #c86437
  """
  if arg == None:
    return arg
  if type(arg) == str:
    if arg == '':
      return arg
    elif arg[0] == '#':
      if re.match(_RERGB, arg):
        return arg
      else:
        raise pyerr.ColorError("bad color string: {0}".format(arg))
    elif arg in _colbyname:
      return _colbyname[arg]
    else:
      raise pyerr.ColorError("bad color name: {0}".format(arg))
  elif type(arg) == tuple:
    try:
      r, g, b = arg
      0+r+g+b
    except:
      raise pyerr.ColorError("bad color arguments: {0}".format(arg))
    if 0 <= r <= 1 and 0 <= g <= 1 and 0 <= b <= 1:
      x = 255.0
      y = 0.5 # to round numbers
      return "#{0:02x}{1:02x}{2:02x}".format(int(r*x+y), 
                                             int(g*x+y), int(b*x+y))
    else:
      return "#{0:02x}{1:02x}{2:02x}".format(r, g, b)
  else:
    return arg
    
def namecolor(arg):
  """Return color name if exist else code color.
  
  Examples:
  >>> print(namecolor(''))
  <BLANKLINE>
  >>> print(namecolor('#ffa500'))
  orange
  >>> print(namecolor('#c0c0c0'))
  #c0c0c0
  >>> print(namecolor((0.2, 0.8, 0.55)))
  #33cc8c
  >>> print(codecolor((200, 100, 55)))
  #c86437
  
  """
  if arg in _colbycode:
    return _colbycode[arg]
  else:
    return codecolor(arg)

def invcolor(arg):
  """Return code of inverse color.
  
  Examples:
  >>> print(invcolor(''))
  <BLANKLINE>
  >>> print(invcolor('orange'))
  #005aff
  >>> print(invcolor('#c0c0c0'))
  #3f3f3f
  >>> print(invcolor((0.2, 0.8, 0.55)))
  #cc3373
  >>> print(invcolor((200, 100, 55)))
  #379bc8
  
  """
  if arg == '':
    return arg
  else:
    return '#{0:06x}'.format(0xffffff-int(codecolor(arg)[1:], 16))
  
if __name__ == '__main__':
  import doctest
  doctest.testmod()

